@@ -6,12 +6,25 @@ import datetime
 import functools
 import warnings
 
+import requests
+
 import PyTado.interface.api as API
 from PyTado.exceptions import TadoException
 from PyTado.http import DeviceActivationStatus, Http
 
 
 def deprecated(new_func_name):
+    """
+    A decorator to mark functions as deprecated. It will result in a warning being emitted
+    when the function is used, advising the user to use the new function instead.
+
+    Args:
+        new_func_name (str): The name of the new function that should be used instead.
+
+    Returns:
+        function: A decorator that wraps the deprecated function and emits a warning.
+    """
+
     def decorator(func):
         @functools.wraps(func)
         def wrapper(*args, **kwargs):
@@ -31,18 +44,35 @@ def deprecated(new_func_name):
 class Tado:
     """Interacts with a Tado thermostat via public API.
 
-    Example usage: t = Tado('me@somewhere.com', 'mypasswd')
+    Example usage: t = Tado()
+                   t.device_activation() # Activate device
                    t.get_climate(1) # Get climate, zone 1.
     """
 
     def __init__(
         self,
-        http_session=None,
+        token_file_path: str | None = None,
+        saved_refresh_token: str | None = None,
+        http_session: requests.Session | None = None,
         debug: bool = False,
     ):
-        """Class Constructor"""
+        """
+        Initializes the interface class.
+
+        Args:
+            token_file_path (str | None, optional): Path to a file which will be used to persist
+                                                    the refresh_token token. Defaults to None.
+            saved_refresh_token (str | None, optional): A previously saved refresh token.
+                                                        Defaults to None.
+            http_session (requests.Session | None, optional): An optional HTTP session to use for
+                                                              requests (can be used in unit tests).
+                                                              Defaults to None.
+            debug (bool, optional): Flag to enable or disable debug mode. Defaults to False.
+        """
 
         self._http = Http(
+            token_file_path=token_file_path,
+            saved_refresh_token=saved_refresh_token,
             http_session=http_session,
             debug=debug,
         )
@@ -50,16 +80,12 @@ class Tado:
         self._debug = debug
 
     def __getattr__(self, name):
-        """Delegiert den Aufruf von Methoden an die richtige API-Client-Implementierung."""
+        """Delegate the called method to api implementation (hops_tado.py or my_tado.py)."""
 
-        if self._api is None:
-            raise TadoException("API is not initialized. Please complete device authentication first.")
+        self._ensure_api_initialized()
 
         return getattr(self._api, name)
 
-    # region Deprecated Methods
-    # pylint: disable=invalid-name
-
     def device_verification_url(self) -> str | None:
         """Returns the URL for device verification."""
         return self._http.device_verification_url
@@ -71,12 +97,30 @@ class Tado:
     def device_activation(self) -> None:
         """Activates the device."""
         self._http.device_activation()
+        self._ensure_api_initialized()
 
-        if self._http.device_activation_status == DeviceActivationStatus.COMPLETED:
-            if self._http.is_x_line:
-                self._api = API.TadoX(http=self._http, debug=self._debug)
+    def get_refresh_token(self) -> str | None:
+        """
+        Retrieve the refresh token from the current api connection.
+
+        Returns:
+            str | None: The current refresh token, or None if not available.
+        """
+        return self._http.refresh_token
+
+    def _ensure_api_initialized(self):
+        """Ensures the API client is initialized."""
+        if self._api is None:
+            if self._http.device_activation_status == DeviceActivationStatus.COMPLETED:
+                if self._http.is_x_line:
+                    self._api = API.TadoX(http=self._http, debug=self._debug)
+                else:
+                    self._api = API.Tado(http=self._http, debug=self._debug)
             else:
-                self._api = API.Tado(http=self._http, debug=self._debug)
+                raise TadoException("API is not initialized. Please complete device authentication first.")
+
+    # region Deprecated Methods
+    # pylint: disable=invalid-name
 
     @deprecated("get_me")
     def getMe(self):
